<?php
/**
 * Administration form to configure FEA.
 */
function fea_settings_form() {
  $form = array();
  $permissions = fea_permission();
  $ids = array_map('_fea_strip_safe_form_id', array_keys(_fea_get_form_ids()));

  $form['form_ids'] = array(
    '#title' => t('Form IDs'),
    '#type' => 'fieldset',
  );
  $form['form_ids']['fea__display_form_id'] = array(
    '#type' => 'checkbox',
    '#title' => t('Display all form ids for users with the %perm permission.', array(
      '%perm' => $permissions['administer fea settings']['title'],
    )),
    '#default_value' => variable_get('fea__display_form_id', FALSE),
  );

  if (isset($_GET['add'])) {
    drupal_set_message(t('The added form id will not be saved until this form is submitted.'), 'warning');
    $ids[] = $_GET['add'];
  }
  $form['form_ids']['form_ids'] = array(
    '#type' => 'textarea',
    '#title' => t('Enable FEA on these form ids'),
    '#description' => t('Put each form id on a new line. No wildcards allowed.'),
    '#default_value' => implode("\n", $ids),
  );

  return system_settings_form($form);
}

/**
 * Form validate function to explode form ids.
 */
function fea_settings_form_validate(&$form, &$form_state) {
  $ids = array_filter(array_map('trim', explode("\n", $form_state['values']['form_ids'])));
  $form_ids = _fea_get_form_ids();

  foreach ($ids as $index => $id) {
    if (!isset($form_ids[_fea_safe_form_id($id)])) {
      variable_set(_fea_safe_form_id($id), array(
        'form_id' => $id,
        'invert' => FALSE,
        'elements' => array(),
      ));
    }
    else {
      unset($form_ids[_fea_safe_form_id($id)]);
    }
  }

  // Delete remaining form_ids.
  foreach (array_keys($form_ids) as $id) {
    variable_del($id);
  }

  // Don't store this value.
  unset($form_state['values']['form_ids']);
}

/**
 * Administration form to configure field access.
 *
 * @todo: auto enable parent fieldsets
 */
function fea_access_form() {
  $form = array();
  $roles = user_roles();
  $form_ids = _fea_get_form_ids(TRUE);

  foreach ($form_ids as $safe_form_id => $config) {
    $form[$safe_form_id] = array(
      '#type' => 'fieldset',
      '#title' => check_plain($config['form_id']),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#tree' => TRUE,
    );
    $form[$safe_form_id]['form_id'] = array(
      '#type' => 'value',
      '#value' => $config['form_id'],
    );
    $form[$safe_form_id]['invert'] = array(
      '#type' => 'checkbox',
      '#title' => t('Revoke access to selected elements'),
      '#description' => t('Uncheck this to revoke access to all <em>unselected</em> elements'),
      '#default_value' => $config['invert'],
    );

    if (!empty($config)) {
      foreach ($config['elements'] as $element_id => $info) {
        $form[$safe_form_id]['elements'][$element_id]['tree'] = array(
          '#type' => 'value',
          '#value' => $info['tree'],
        );
        $form[$safe_form_id]['elements'][$element_id]['type'] = array(
          '#type' => 'value',
          '#value' => $info['type'],
        );
        $form[$safe_form_id]['elements'][$element_id]['roles'] = array(
          '#title' => t('@tree (@type)', array(
            '@tree' => implode(' > ', $info['tree']),
            '@type' => $info['type'],
          )),
          '#type' => 'checkboxes',
          '#options' => drupal_map_assoc(array_values($roles)),
          '#default_value' => $info['roles'],
        );
      }
    }
    else {
      $form[$safe_form_id]['#description'] = t('No form elements indexed.');
    }
  }

  // Check if all forms are indexed.
  foreach (array_keys($form_ids) as $safe_form_id) {
    if (!isset($form[$safe_form_id])) {
      drupal_set_message(t('Please visit the page with form id %form_id on it to be able to configure the elements here.', array(
        '%form_id' => $safe_form_id,
      )), 'warning', FALSE);
    }
  }

  // No form ids configured?
  if (empty($form_ids)) {
    drupal_set_message(t('No configured form ids. Please add them on the !link page.', array(
      '!link' => l(t('settings'), 'admin/config/user-interface/fea/settings'),
    )), 'error', FALSE);
  }

  return system_settings_form($form);
}
